#include "mlf-driver.h"
#include "mlf-parser.tab.h"

mlf_driver::mlf_driver() : trace_scanning (false), trace_parsing (false){
	formula = NULL;
}

int mlf_driver::parse (const std::string &f){
	file = f;
	scan_begin ();
	yy::mlf_parser parser (*this);
	parser.set_debug_level (trace_parsing);
	int res = parser.parse ();
	scan_end ();
	return res;
}

void mlf_driver::error (const yy::location& l, const std::string& m){
	std::cerr << l << ": " << m << std::endl;
}

void mlf_driver::error (const std::string& m){
	std::cerr << m << std::endl;
}

bdd mlf_driver::variable(int n){
	if(!formula){
		cout << "driver.formula was null!" << endl;
		exit(1);
	}
	return formula->variable(n);
}

bdd mlf_driver::modal(bdd *b, int n, int m){
	if(!formula){
		cout << "driver.formula was null!" << endl;
		exit(1);
	}
	return formula->modal(b, n, m);
}


void mlf_driver::set_formula_bdd(bdd b){
	if(!formula){
		cout << "driver.formula was null!" << endl;
		exit(1);
	}
	formula->set_bdd(b);
}

void mlf_driver::extract_formula(IFormula* &ptr){
	ptr = formula;
	formula = NULL;
}		

mlf_driver::~mlf_driver(){
	if(formula!=NULL)
		delete formula;
}

