%skeleton "lalr1.cc"
%require "2.4"
%defines
%define parser_class_name "mlf_parser"

%code requires {
	#include <string>
	#include "bdd.h"
	#include "../formulas/GML_formula.h"
	#include "../formulas/PML_formula.h"
	#include "../formulas/rational.h"

	using namespace std;

	class mlf_driver;
	#define YYSTYPE ParserType
	
	struct ParserType
{
	int number;
	struct fraction {
		int numerator;
		int denominator;
	} fraction;
	bdd binarydd;
};	
}



// Parsing context
%parse-param { mlf_driver& driver}
%lex-param { mlf_driver& driver}

%locations
%initial-action {
	// Initialize the initial location.
	@$.begin.filename = @$.end.filename = &driver.file;
};

%debug
%error-verbose

%code {
	#include "mlf-driver.h"
}

/* operators and precendences */
     %nonassoc  IFF
     %right  IMP
     %left  OR
     %left  AND
     %nonassoc <number> GMDIA GMBOX 
     %nonassoc <fraction> PMDIA PMBOX 
     %nonassoc <binarydd> PVAR
     %nonassoc NOT TRUE FALSE

     /* auxiliary */
     %token END			0	"end of file"
     %token LPAREN		
     %token RPAREN
     
     %token GML
     %token PML
     //%token PMLI
     
    // %printer    { debug_stream () << $$; } "identifier"
     %destructor { delete &$$; } "identifier"

	%type <binarydd> formula

%%
input: 
	spec
 	formula {driver.set_formula_bdd($2); }
	;
	
spec:	
	GML {driver.set_formula_gml();}
	| PML {driver.set_formula_pml();} 
	//| PMLI {}
	;
    
formula:   
       formula IFF formula { $$ = bdd_biimp($1,$3); }
     | formula IMP formula { $$ = bdd_imp($1,$3); }
     | formula  OR formula { $$ = bdd_or($1,$3); } 
     | formula AND formula { $$ = bdd_and($1,$3); }
     
     | GMDIA formula      { bdd* b = new bdd($2); $$ = driver.modal(b, $1, 0); } 
     | GMBOX formula      { bdd* b = new bdd(bdd_not($2)); $$ = bdd_not(driver.modal(b, $1, 0)); } // box is not-dia-not 

     | PMDIA formula      { bdd* b = new bdd($2); $$ = driver.modal(b, $1.numerator, $1.denominator); } 
     | PMBOX formula      { bdd* b = new bdd(bdd_not($2)); $$ = bdd_not(driver.modal(b, $1.numerator, $1.denominator)); }
     
     | NOT formula         { $$ = bdd_not($2); }
     | PVAR                { $$ = $$; }
	 | TRUE                { $$ = bdd_true(); }
	 | FALSE               { $$ = bdd_false(); }
     | LPAREN formula RPAREN { $$ = $2; }
     ;
%%

   
void yy::mlf_parser::error (const yy::mlf_parser::location_type& l, const std::string& m){
	driver.error (l, m);
}



	
