%{
# include <cstdlib>
# include <errno.h>
# include <limits.h>
# include <string>
# include "mlf-driver.h"
# include "mlf-parser.tab.h"

/* Added since it fixes compile errors */
#define YY_NEW_FILE yyrestart(yyin  )

#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
	yyleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
	(yy_c_buf_p) = yy_cp;
/* ----------------------- */

/* Work around an incompatibility in flex (at least versions
   2.5.31 through 2.5.33): it generates code that does
   not conform to C89.  See Debian bug 333231
   <http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=333231>.  */
# undef yywrap
# define yywrap() 1

/* By default yylex returns int, we use token_type.
   Unfortunately yyterminate by default returns 0, which is
   not of token_type.  */
#define yyterminate() return token::END
%}

%option noyywrap nounput batch debug



%{
# define YY_USER_ACTION  yylloc->columns (yyleng);
%}

%{
  typedef yy::mlf_parser::token token;
%}


%%
[ \t\n]+ /*eat white space*/
"Logic:GML"				{ return token::GML; }
"Logic:PML"				{ return token::PML; }
"Logic: GML"			{ return token::GML; }
"Logic: PML"			{ return token::PML; }
"&" 					{ return token::AND; }
"->"   					{ return token::IMP; }
"<->"					{ return token::IFF; }
"<"[0-9]+">"			{	if(driver.get_logic()!='g')
     							driver.error(*yylloc, "Invalid operator type for specified logic!");
     						else {
     							yylval->number = atoi(yytext + 1); 
     							return token::GMDIA; 
     						}
     					}
"["[0-9]+"]"			{	if(driver.get_logic()!='g')
     							driver.error(*yylloc, "Invalid operator type for specified logic!");
     						else {
     							yylval->number = atoi(yytext + 1); 
     							return token::GMBOX; 
     						}
     					}
"<"[0-9]+"/"[0-9]+">"	{ 	if(driver.get_logic()!='p')
     							driver.error(*yylloc, "Invalid operator type for specified logic!");
     						else {
       							(yylval->fraction).numerator = atoi(yytext + 1);
     							/*TODO: find more elegant way to perform this - skipping through until "/" */
								int a = 1;
								int b = atoi(yytext + 1);
								while(b > 0){
									b = b/10;
									a++;
								}
     							(yylval->fraction).denominator = atoi(yytext + a + 1);
     							return token::PMDIA;
     						}
     					}
"["[0-9]+"/"[0-9]+"]"	{ 	if(driver.get_logic()!='p')
     							driver.error(*yylloc, "Invalid operator type for specified logic!");
     						else {
     							(yylval->fraction).numerator = atoi(yytext + 1);
								int a = 1;
								int b = atoi(yytext + 1);
								while(b > 0){
									b = b/10;
									a++;
								}
     							(yylval->fraction).denominator = atoi(yytext + a + 1);
     							return token::PMBOX;
     						}
     					}
"v"						{ return token::OR; }
"~"						{ return token::NOT; }
p[0-9]+					{ yylval->binarydd = driver.variable(atoi(yytext + 1)); return token::PVAR; }
"("						{ return token::LPAREN; }
")"						{ return token::RPAREN; }
"true"					{ return token::TRUE; }
"false"					{ return token::FALSE; }
.          driver.error (*yylloc, "invalid character!");
%%

void mlf_driver::scan_begin (){
  yy_flex_debug = trace_scanning;
  if (file == "-")
    yyin = stdin;
  else if (!(yyin = fopen (file.c_str (), "r")))
    {
      error (std::string ("cannot open ") + file);
      exit (1);
    }
}

void mlf_driver::scan_end (){
  fclose (yyin);
}

