#include "GML_premise.h"

GML_Premise::GML_Premise() : Premise<int>(){
}

GML_Premise::GML_Premise(int _n, int _m, int* _a, int *_b) : Premise<int>(_n,_m,_a,_b){
//test_counter = 0;
}


/* Deprecated Functions which searched for solutions to the side condition first.
vector<Conclusion> GML_Premise::construct_conclusions(){
	double limit = bound();
	vector<Conclusion> set;
	glp_prob *side_condition = glp_create_prob();
	
	// Parameters while solving with glpk
	glp_iocp* parameters = new glp_iocp;
	glp_init_iocp(parameters);
	parameters->presolve = GLP_ON;
	parameters->msg_lev = GLP_MSG_ERR;
	
	// Load numbers into LP
	load_side_condition(side_condition, limit);

	// output for testing.
	//glp_write_lp(side_condition, NULL, "output.txt");

	// Solve depth first	
	solve_side_condition(side_condition, parameters, 0, 0, 0, limit, set);
	
	//Clean-up
	delete parameters;
	glp_delete_prob(side_condition);
	
	return set;
}

void GML_Premise::load_side_condition(glp_prob* sc, double limit){
	glp_set_obj_dir(sc, GLP_MIN);
	glp_add_rows(sc, 1);
	glp_set_row_bnds(sc, 1, GLP_LO, 1.0, 0.0);

	glp_add_cols(sc, n+m);
	
	for(int i=1; i < (n+m+1); i++){
		glp_set_col_kind(sc, i, GLP_IV); //integer variables
		glp_set_col_bnds(sc, i,  GLP_DB, 0, limit);
	}
	
	// Arrays for inputting into glpk
	double ar[n+m+1];
	int ia[n+m+1];
	int ja[n+m+1];
	
	for(int i=1; i < n+m+1; i++){
   		ia[i]=1;
     	ja[i]=i;
     }
     
    int counter=1;	
	for(int i=0; i < n; i++){
		ar[counter]= a[i]+1;
		counter++;
	}

	for(int j=0; j < m; j++){
		ar[counter]=(-b[j]);
		counter++;
	}

	// Set objective function	
	for(int i=1; i < n+m+1; i++)
		glp_set_obj_coef(sc, i, ar[i]);
		
	glp_load_matrix(sc, n+m, ia, ja, ar);
}

void GML_Premise::solve_side_condition(glp_prob* sc, glp_iocp* parameters, int r_or_s, int index, int new_bound, double limit, vector<Conclusion>& set){
	int r[n];
	int s[m];
	int old_bound;
	Conclusion conc;
	
	if(r_or_s == 1){
		if(new_bound < 1)
			return;
		// add r_i < bound    
		old_bound = glp_get_col_ub(sc, index+1);
		if(new_bound == 1)
			glp_set_col_bnds(sc, index+1,  GLP_FX, 0, 0);
		else			
			glp_set_col_bnds(sc, index+1,  GLP_DB, 0, new_bound-1); // Minus once since need strictly less than. Glpk is less or equal.
	}
	
	if(r_or_s == 2){
		if(new_bound+1 > limit)
			return;
		// add bound s_j > bound
		old_bound = glp_get_col_lb(sc, index+n+1);
		if(new_bound+1 == limit)
			glp_set_col_bnds(sc, index+n+1,  GLP_FX, limit, limit);
		else
			glp_set_col_bnds(sc, index+n+1,  GLP_DB, new_bound+1, limit);
	}
	
	//Test output 
	//char str[15];
	//sprintf(str, "output%d.txt", test_counter);
	//glp_write_lp(sc, NULL, str);	
	//test_counter++;
	
			
	// Solve
	int result = glp_intopt(sc, parameters);
	
	if(result == 0) { //if no errors from solver
		if(glp_mip_status(sc)==GLP_OPT || glp_mip_status(sc)==GLP_FEAS){ // if feasible
		
		// Load r and s from solution
		for(int i=0; i < n; i++)
			r[i] = static_cast<int>(glp_mip_col_val(sc, i+1));
			
		for(int j=0; j < m; j++)
			s[j] = static_cast<int>(glp_mip_col_val(sc, j+n+1));

 		// do all valuations
 		all_valuations(parameters, r, s, conc);
 	

 		//if result isnt in list of conclusions then add it
 		bool already_have = false;
 		for(unsigned int i=0; i < set.size(); i++)
 			if( conc == set[i])
 				already_have = true;
 		if(!already_have)
			set.push_back(conc);
 			
		// Recursively branch and solve the others - depth first
		for(int i=0; i < n; i++)
			solve_side_condition(sc, parameters, 1, i, r[i], limit, set);
		for(int j=0; j < m; j++)
			solve_side_condition(sc, parameters, 2, j, s[j], limit, set);
		}
	}
	else if (result != GLP_ENOPFS) { // if not a feasibility error
		cout << "glpk error: " << result << endl;
		exit(1);
	}
	//remove bounds (go back up before branching down again).
	if(r_or_s == 1)
		glp_set_col_bnds(sc, index+1,  GLP_DB, 0, old_bound);
	
	if(r_or_s == 2)
		glp_set_col_bnds(sc, index+n+1,  GLP_DB, old_bound, limit);
}

void GML_Premise::all_valuations(glp_iocp* parameters, int* r, int* s, Conclusion& result){
//Testing output
cout << "====================" << endl;
	for(int i=0; i < n; i++)
		cout << "r[" << i << "] = " << r[i] << endl;
	
	for(int j=0; j < m; j++)
		cout << "s[" << j << "] = " << s[j] << endl;


glp_prob *conclusion = glp_create_prob();
glp_set_obj_dir(conclusion, GLP_MIN);
glp_add_rows(conclusion, 2);
glp_set_row_bnds(conclusion, 1, GLP_LO, 1, 0); // strictly greater than zero

glp_add_cols(conclusion, n+m);

for(int i=1; i < (n+m+1); i++)
	glp_set_col_kind(conclusion, i, GLP_BV);

// Arrays for inputting into glpk
	double ar[2*(n+m)+1];
	int ia[2*(n+m)+1];
	int ja[2*(n+m)+1];
	
	for(int i=1; i < n+m+1; i++){
   		ia[i]=1;
   		ia[n+m+i]=2;
     	ja[i]=i;
     	ja[n+m+i]=i;
     }
     
    int counter=1;	
	for(int i=0; i < n; i++){
		ar[counter]= r[i];
		counter++;
	}

	for(int j=0; j < m; j++){
		ar[counter]=(-s[j]);
		counter++;
	}
	
	for(int i=0; i < n+m; i++){
		ar[counter]=pow(2,i);
		counter++;
	}

	// Set objective function	
	for(int i=1; i < n+m+1; i++)
		glp_set_obj_coef(conclusion, i, pow(2,i-1));
		
	glp_load_matrix(conclusion, 2*(n+m), ia, ja, ar);
	
		glp_set_row_bnds(conclusion, 2, GLP_LO, 0, 0);
		
	// output for testing
	//glp_write_lp(conclusion, NULL, "valuations.txt");	
	
	solve_conclusion_lp(conclusion, parameters, 0, result);
	
	glp_delete_prob(conclusion);
}

void GML_Premise::solve_conclusion_lp(glp_prob* conclusion, glp_iocp* parameters, int min_limit, Conclusion& conc){
	glp_set_row_bnds(conclusion, 2, GLP_LO, min_limit, 0);
	
	int result = glp_intopt(conclusion, parameters);
	
	if(result == 0){//if no errors from solver
		if(glp_mip_status(conclusion)==GLP_OPT || glp_mip_status(conclusion)==GLP_FEAS){ // if feasible
			int p[n+m];
			for(int i=0; i<n+m; i++)
				p[i]=glp_mip_col_val(conclusion, i+1);
				//cout << glp_mip_col_val(conclusion, i+1);
			Clause c(n+m,p);
			conc.push_back(c);
		
			solve_conclusion_lp(conclusion, parameters, glp_mip_obj_val(conclusion)+1, conc);
		}
	}
	else if (result != GLP_ENOPFS) { // if not a feasibility error
		cout << "glpk error: " << result << endl;
		exit(1);	
	}
}


double GML_Premise::bound(){
	return 2;
	//return pow(2,6 * sizeoflp(n, m, a, b) * (n+m) * (n+m) * (n+m));
}

END OF DEPRECATED FUNCTIONS */
