#include "premise.h"

template<class T>
Premise<T>::Premise(){
	n = 0;
	m = 0;
	total_valuations = 1 << (n+m);
	a = NULL;
	b = NULL;
}

template<class T>
Premise<T>::Premise(int _n, int _m, T* _a, T* _b){
	n =_n;
	m =_m;
	if (n+m > 30) {
		cout << "Too many (>30) modal formulae in node." << endl;
		exit(1);
	}
	total_valuations = 1 << (n+m);
	
	if(n)
		a = new T [n];
	else
		a = NULL;
	if(m)
		b = new T [m];
	else
		b = NULL;
	
	for(int i=0; i < n; i++)
		a[i]=_a[i];
	for(int j=0; j < m; j++)
		b[j]=_b[j];
}

template<class T>
Premise<T>::Premise(const Premise<T>& p){
	n = p.n;
	m = p.m;
	total_valuations = p.total_valuations;
	
	if(n)
		a = new T [n];
	else
		a = NULL;
		
	if(m)
		b = new T [m];
	else
		b = NULL;
	
	for(int i=0; i < n; i++)
		a[i]=p.a[i];
	for(int j=0; j < m; j++)
		b[j]=p.b[j];
}

template<class T>
Premise<T>& Premise<T>::operator=(const Premise<T>& p){
	if(this != &p){
		if(a) 
			delete [] a;
		if(b)
			delete [] b;
		n = p.n;
		m = p.m;
		total_valuations = p.total_valuations;

		if(n)
			a = new T [n];
		else
			a = NULL;
		if(m)
			b = new T [m];
		else
			b = NULL;
			
		for(int i=0; i < n; i++)
			a[i]=p.a[i];
		for(int j=0; j < m; j++)
			b[j]=p.b[j];
	}
	return *this;
}

template<class T>
Premise<T>::~Premise(){
	if(a)
		delete [] a;
	if(b)
		delete [] b;
}

template<class T>
T Premise<T>::get_a_i(int i){
	if(a)
		return a[i];
	return -1;
}

template<class T>
T Premise<T>::get_b_i(int i){
	if(b)
		return b[i];
	return -1;
}

template class Premise<int>;
template class Premise<Rational>;
